#include "svg.h"
#include <iostream>
#include <string>
#include <algorithm>

using namespace std;

void svg_begin(double width, double height) {
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg width='" << width << "' height='" << height << "' "
        << "viewBox='0 0 " << width << " " << height << "' "
        << "xmlns='http://www.w3.org/2000/svg'>\n";
}

void svg_end() {
    cout << "</svg>\n";
}

void svg_text(double left, double baseline, const string& text) {
    cout << "<text x='" << left << "' y='" << baseline << "'>"
        << text << "</text>\n";
}

void svg_rect(double x, double y, double width, double height,
    const string& stroke, const string& fill) {
    cout << "<rect x='" << x << "' y='" << y << "' "
        << "width='" << width << "' height='" << height << "' "
        << "stroke='" << stroke << "' fill='" << fill << "'/>\n";
}

void show_histogram_svg(const vector<size_t>& bins) {
    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto MAX_BIN_WIDTH = IMAGE_WIDTH - TEXT_WIDTH - 10;

    svg_begin(IMAGE_WIDTH, IMAGE_HEIGHT);

    //       
    size_t max_count = *max_element(bins.begin(), bins.end());
    double scaling_factor = max_count > 0 ?
        static_cast<double>(MAX_BIN_WIDTH) / max_count : 1.0;

    double top = 0;
    for (size_t bin : bins) {
        const double bin_width = bin * scaling_factor;
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, to_string(bin));
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT,
            "#2ecc71", "#27ae60"); //   (   )
        top += BIN_HEIGHT;
    }

    svg_end();
}